/*
 AT Mini Matrix Ctrl
 Version: 0.4
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 20/02/2013
 Last Modified: 11/05/2013
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define start_time 0
#define end_time 1
#define delay_time 2
#define start_location 4
#define end_location 3
#define animation_speed 5
#define display_type 6

#define A 0
#define B 1
#define C 2
#define D 3
#define E 4
#define F 5
#define G 6
#define H 7
#define I 8
#define J 9
#define K 10
#define L 11
#define M 12
#define N 13
#define O 14
#define P 15
#define Q 16
#define R 17
#define S 18
#define T 19
#define U 20
#define V 21
#define W 22
#define X 23
#define Y 24
#define Z 25

// Text array
prog_uchar ledLetters[26][8] PROGMEM = {
	{24,8,20,20,20,28,34,119}, // A
	{124,34,34,60,34,34,34,124}, // B
	{30,34,64,64,64,64,34,28}, // C
	{120,36,34,34,34,34,36,120},
	{126,34,40,56,40,32,34,126},
	{126,34,40,56,40,32,32,112},
	{30,34,64,64,71,66,34,28},
	{119,34,34,62,34,34,34,119},
	{62,8,8,8,8,8,8,62},
	{30,4,4,4,68,68,68,56},
	{119,34,36,40,56,36,34,115},
	{112,32,32,32,32,34,34,126},
	{119,54,54,42,42,34,34,119},
	{231,98,82,82,74,74,70,230},
	{28,34,65,65,65,65,34,28},
	{124,34,34,34,60,32,32,112}, // P
	{28,34,65,65,65,77,34,29},
	{124,34,34,34,60,36,34,113},
	{58,70,64,60,2,2,98,92},
	{127,73,8,8,8,8,8,28},
	{119,34,34,34,34,34,34,28},
	{231,66,66,36,36,36,24,24},
	{119,34,20,8,8,20,34,119},
	{119,34,20,8,8,20,34,119},
	{119,34,20,20,8,8,8,28},
	{62,34,4,8,8,16,34,62} // Z
};

// Sleep mode - power save
void system_sleep_pwr_save(void) {
	set_sleep_mode(SLEEP_MODE_PWR_SAVE); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Sleep mode - power down
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup the ATmega
void setup(void) {
	// Set Outputs
	DDRB |= (1<<PB5) | (1<<PB4) | (1<<PB3) | (1<<PB2) | (1<<PB0);
	DDRC |= (1<<PC3) | (1<<PC2) | (1<<PC1) | (1<<PC0);
	DDRD |= (1<<PD7) | (1<<PD6) | (1<<PD5) | (1<<PD4) |  (1<<PD3) | (1<<PD2) | (1<<PD1);
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= (1<<PB1);
	PORTC |= (1<<PC5) | (1<<PC4);
	
	// Enable the 32.768KHz crystal as a RTC
	TIMSK2 = 0; // Disable the Timer/Counter2 interrupts by clearing OCIE2x and TOIE2
	ASSR = (1<<AS2); // Select clock source by setting AS2 as appropriate
	TCNT2 = 0;// Write new values to TCNT2, OCR2x, and TCCR2x
	OCR2A = 0;
	OCR2B = 0;
	TCCR2A = 0;
	TCCR2B = (1<<CS22) | (1<<CS21) | (1<<CS20); // 1024 prescaler for 8 seconds overflow on 32.768KHz crystal
	//TCCR2B = (1<<CS22) | (1<<CS20); // 128 prescaler for 1 second overflow on 32.768KHz crystal
	while ((ASSR & ((1<<TCN2UB) | (1<<OCR2AUB) | (1<<OCR2BUB) | (1<<TCR2AUB) | (1<<TCR2BUB))) != 0); // To switch to asynchronous operation: Wait for TCN2xUB, OCR2xUB, and TCR2xUB ASSR
	TIFR2 = 0; // Clear the Timer/Counter2 Interrupt Flags
	TIMSK2 |= (1<<TOIE2); // Enable overflow interrupt
	
	// After a Power-up Reset or wake-up from Power-down or Standby mode, the user should be aware of the fact 
	// that this Oscillator might take as long as one second to stabilize
	_delay_ms(1000);
	
	sei(); // Turn on interrupts
}
